package middleware

import (
	"context"
	"fmt"

	"git.perx.ru/perxis/perxis-go/id"
	"git.perx.ru/perxis/perxis-go/pkg/collaborators"
	logzap "git.perx.ru/perxis/perxis-go/zap"
	"go.uber.org/zap"
)

type loggingMiddleware struct {
	logger *zap.Logger
	next   collaborators.Collaborators
}

func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next collaborators.Collaborators) collaborators.Collaborators {
		return &loggingMiddleware{
			next:   next,
			logger: logger.With(logzap.Component("Collaborators")),
		}
	}
}

func (m *loggingMiddleware) Set(ctx context.Context, spaceId, subject, role string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(collaborators.EventSet),
		logzap.Object(id.NewSpaceId(spaceId)),
	)

	err = m.next.Set(ctx, spaceId, subject, role)
	if err != nil {
		logger.Error(fmt.Sprintf("Failed to set user '%s' as a collaborator with role '%s'", subject, role), zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info(fmt.Sprintf("User '%s' assigned to role '%s'", subject, role), logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) Get(ctx context.Context, spaceId, subject string) (role string, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	role, err = m.next.Get(ctx, spaceId, subject)
	if err != nil {
		logger.Error(fmt.Sprintf("Failed to get role for collaborator %s", subject), zap.Error(err))
		return
	}

	return role, err
}

func (m *loggingMiddleware) Remove(ctx context.Context, spaceId, subject string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(collaborators.EventRemove),
		logzap.Object(id.NewSpaceId(spaceId)),
	)

	err = m.next.Remove(ctx, spaceId, subject)
	if err != nil {
		logger.Error(fmt.Sprintf("Failed to remove user '%s' from space", subject), zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info(fmt.Sprintf("User '%s' removed from space", subject), logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) ListCollaborators(ctx context.Context, spaceId string) (collaborators []*collaborators.Collaborator, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	collaborators, err = m.next.ListCollaborators(ctx, spaceId)
	if err != nil {
		logger.Error("Failed to list collaborators", zap.Error(err))
		return
	}

	return collaborators, err
}

func (m *loggingMiddleware) ListSpaces(ctx context.Context, subject string) (spaces []*collaborators.Collaborator, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	spaces, err = m.next.ListSpaces(ctx, subject)
	if err != nil {
		logger.Error("Failed to list spaces", zap.Error(err))
		return
	}

	return spaces, err
}
