package middleware

import (
	"context"

	"git.perx.ru/perxis/perxis-go/id"
	"git.perx.ru/perxis/perxis-go/pkg/roles"
	logzap "git.perx.ru/perxis/perxis-go/zap"
	"go.uber.org/zap"
)

type loggingMiddleware struct {
	logger *zap.Logger
	next   roles.Roles
}

func LoggingMiddleware(logger *zap.Logger) Middleware {
	return func(next roles.Roles) roles.Roles {
		return &loggingMiddleware{
			next:   next,
			logger: logger.With(logzap.Component("Roles")),
		}
	}
}

func (m *loggingMiddleware) Create(ctx context.Context, role *roles.Role) (created *roles.Role, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(roles.EventCreate),
	)

	created, err = m.next.Create(ctx, role)
	if err != nil {
		logger.Error("Failed to create", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog), logzap.Object(role))
		return
	}

	logger.Info("Role created", logzap.Channels(logzap.Userlog), logzap.Object(created))

	return created, err
}

func (m *loggingMiddleware) Get(ctx context.Context, spaceId, roleId string) (role *roles.Role, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Object(id.NewRoleId(spaceId, roleId)),
	)

	role, err = m.next.Get(ctx, spaceId, roleId)
	if err != nil {
		logger.Error("Failed to get", zap.Error(err))
		return
	}

	return role, err
}

func (m *loggingMiddleware) List(ctx context.Context, spaceId string) (roles []*roles.Role, err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
	)

	roles, err = m.next.List(ctx, spaceId)
	if err != nil {
		logger.Error("Failed to list", zap.Error(err))
		return
	}

	return roles, err
}

func (m *loggingMiddleware) Update(ctx context.Context, role *roles.Role) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(roles.EventUpdate),
		logzap.Object(role),
	)

	err = m.next.Update(ctx, role)
	if err != nil {
		logger.Error("Failed to update", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Role updated", logzap.Channels(logzap.Userlog))

	return err
}

func (m *loggingMiddleware) Delete(ctx context.Context, spaceId, roleId string) (err error) {
	logger := m.logger.With(
		logzap.Caller(ctx),
		logzap.Event(roles.EventDelete),
		logzap.Object(id.NewRoleId(spaceId, roleId)),
	)

	err = m.next.Delete(ctx, spaceId, roleId)
	if err != nil {
		logger.Error("Failed to delete", zap.Error(err), logzap.Channels(logzap.Userlog, logzap.Syslog))
		return
	}

	logger.Info("Role deleted", logzap.Channels(logzap.Userlog))

	return err
}
